"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _symbol = require("babel-runtime/core-js/symbol");

var _symbol2 = _interopRequireDefault(_symbol);

var _graphql = require("graphql");

var _queryFromResolveData = require("../queryFromResolveData");

var _queryFromResolveData2 = _interopRequireDefault(_queryFromResolveData);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var nullableIf = function nullableIf(condition, Type) {
  return condition ? Type : new _graphql.GraphQLNonNull(Type);
};

var defaultPgColumnFilter = function defaultPgColumnFilter(_attr, _build, _context) {
  return true;
};

exports.default = function PgColumnsPlugin(builder, _ref) {
  var inflection = _ref.pgInflection,
      _ref$pgColumnFilter = _ref.pgColumnFilter,
      pgColumnFilter = _ref$pgColumnFilter === undefined ? defaultPgColumnFilter : _ref$pgColumnFilter;

  builder.hook("GraphQLObjectType:fields", function (fields, build, context) {
    var extend = build.extend,
        pgGetGqlTypeByTypeId = build.pgGetGqlTypeByTypeId,
        introspectionResultsByKind = build.pgIntrospectionResultsByKind,
        sql = build.pgSql,
        pg2gql = build.pg2gql,
        getAliasFromResolveInfo = build.getAliasFromResolveInfo,
        pgTweakFragmentForType = build.pgTweakFragmentForType;
    var _context$scope = context.scope,
        isPgRowType = _context$scope.isPgRowType,
        isPgCompoundType = _context$scope.isPgCompoundType,
        table = _context$scope.pgIntrospection,
        fieldWithHooks = context.fieldWithHooks;

    if (!(isPgRowType || isPgCompoundType) || !table || table.kind !== "class") {
      return fields;
    }
    return extend(fields, introspectionResultsByKind.attribute.filter(function (attr) {
      return attr.classId === table.id;
    }).filter(function (attr) {
      return pgColumnFilter(attr, build, context);
    }).reduce(function (memo, attr) {
      /*
        attr =
          { kind: 'attribute',
            classId: '6546809',
            num: 21,
            name: 'upstreamName',
            description: null,
            typeId: '6484393',
            isNotNull: false,
            hasDefault: false }
        */
      var fieldName = inflection.column(attr.name, table.name, table.namespaceName);
      if (memo[fieldName]) {
        throw new Error(`Two columns produce the same GraphQL field name '${fieldName}' on class '${table.namespaceName}.${table.name}'; one of them is '${attr.name}'`);
      }
      memo[fieldName] = fieldWithHooks(fieldName, function (_ref2) {
        var getDataFromParsedResolveInfoFragment = _ref2.getDataFromParsedResolveInfoFragment,
            addDataGenerator = _ref2.addDataGenerator;

        var ReturnType = pgGetGqlTypeByTypeId(attr.typeId) || _graphql.GraphQLString;
        addDataGenerator(function (parsedResolveInfoFragment) {
          var alias = parsedResolveInfoFragment.alias;

          if (attr.type.type === "c") {
            return {
              pgQuery: function pgQuery(queryBuilder) {
                // json_build_object
                /*
                queryBuilder.select(
                  sql.identifier(queryBuilder.getTableAlias(), attr.name),
                  alias
                );
                */
                var resolveData = getDataFromParsedResolveInfoFragment(parsedResolveInfoFragment, ReturnType);
                var jsonBuildObject = (0, _queryFromResolveData2.default)(sql.identifier((0, _symbol2.default)()), // Ignore!
                sql.fragment`(${queryBuilder.getTableAlias()}.${sql.identifier(attr.name)})`, // The brackets are necessary to stop the parser getting confused, ref: https://www.postgresql.org/docs/9.6/static/rowtypes.html#ROWTYPES-ACCESSING
                resolveData, { onlyJsonField: true });
                queryBuilder.select(jsonBuildObject, alias);
              }
            };
          } else {
            return {
              pgQuery: function pgQuery(queryBuilder) {
                queryBuilder.select(pgTweakFragmentForType(sql.fragment`${queryBuilder.getTableAlias()}.${sql.identifier(attr.name)}`, attr.type), alias);
              }
            };
          }
        });
        return {
          description: attr.description,
          type: nullableIf(!attr.isNotNull, ReturnType),
          resolve: function resolve(data, _args, _context, resolveInfo) {
            var alias = getAliasFromResolveInfo(resolveInfo);
            return pg2gql(data[alias], attr.type);
          }
        };
      });
      return memo;
    }, {}));
  });
  builder.hook("GraphQLInputObjectType:fields", function (fields, build, context) {
    var extend = build.extend,
        pgGetGqlInputTypeByTypeId = build.pgGetGqlInputTypeByTypeId,
        introspectionResultsByKind = build.pgIntrospectionResultsByKind;
    var _context$scope2 = context.scope,
        isPgRowType = _context$scope2.isPgRowType,
        isPgCompoundType = _context$scope2.isPgCompoundType,
        isPgPatch = _context$scope2.isPgPatch,
        table = _context$scope2.pgIntrospection,
        pgAddSubfield = _context$scope2.pgAddSubfield;

    if (!(isPgRowType || isPgCompoundType) || !table || table.kind !== "class") {
      return fields;
    }
    return extend(fields, introspectionResultsByKind.attribute.filter(function (attr) {
      return attr.classId === table.id;
    }).filter(function (attr) {
      return pgColumnFilter(attr, build, context);
    }).reduce(function (memo, attr) {
      var fieldName = inflection.column(attr.name, table.name, table.namespaceName);
      if (memo[fieldName]) {
        throw new Error(`Two columns produce the same GraphQL field name '${fieldName}' on input class '${table.namespaceName}.${table.name}'; one of them is '${attr.name}'`);
      }
      memo[fieldName] = pgAddSubfield(fieldName, attr.name, attr.type, {
        description: attr.description,
        type: nullableIf(isPgPatch || !attr.isNotNull || attr.hasDefault, pgGetGqlInputTypeByTypeId(attr.typeId) || _graphql.GraphQLString)
      });
      return memo;
    }, {}));
  });
};
//# sourceMappingURL=PgColumnsPlugin.js.map